@@ -3,13 +3,17 @@ package Wasm;
 use strict;
 use warnings;
 use 5.008001;
+use Ref::Util qw( is_plain_arrayref );
 use Carp ();
 
 # ABSTRACT: Write Perl extensions using Wasm
-our $VERSION = '0.07'; # VERSION
+our $VERSION = '0.08'; # VERSION
 
 
-my $store;
+our %WASM;
+my $linker;
+my %inst;
+my @keep;
 
 sub import
 {
@@ -26,8 +30,10 @@ sub import
   my $api;
   my $exporter;
   my @module;
-  my @imports;
   my $package = $caller;
+  my $file    = $fn;
+
+  my @global;
 
   while(@_)
   {
@@ -58,7 +64,8 @@ sub import
         $path = 'undef' unless defined $path;
         Carp::croak("no such file $path");
       }
-      @module = (file => "$path");
+      $file = "$path";
+      @module = (file => $file);
     }
     elsif($key eq '-self')
     {
@@ -76,7 +83,8 @@ sub import
       }
       else
       {
-        @module = (file => shift @maybe);
+        $file = shift @maybe;
+        @module = (file => $file);
       }
     }
     elsif($key eq '-exporter')
@@ -87,9 +95,20 @@ sub import
     {
       $package = shift;
     }
+    elsif($key eq '-global')
+    {
+      if(is_plain_arrayref $_[0])
+      {
+        push @global, shift;
+      }
+      else
+      {
+        Carp::croak("-global should be an array reference");
+      }
+    }
     elsif($key eq '-imports')
     {
-      @imports = @{ shift() };
+      Carp::croak("-imports was removed in Wasm.pm 0.08");
     }
     else
     {
@@ -97,17 +116,141 @@ sub import
     }
   }
 
+  require Wasm::Wasmtime;
+  $linker ||= do {
+    my $linker = Wasm::Wasmtime::Linker->new(
+      Wasm::Wasmtime::Store->new(
+        Wasm::Wasmtime::Engine->new(
+          Wasm::Wasmtime::Config
+            ->new
+            ->wasm_multi_value(1),
+        ),
+      ),
+    );
+
+    $linker->allow_shadowing(0);
+
+    $linker->define_wasi(
+      Wasm::Wasmtime::WasiInstance->new(
+        $linker->store,
+        'wasi_snapshot_preview1',
+        Wasm::Wasmtime::WasiConfig
+          ->new
+          ->set_argv(@ARGV)
+          ->inherit_env
+          ->inherit_stdin
+          ->inherit_stdout
+          ->inherit_stderr
+          #->preopen_dir ?
+      )
+    );
+    $WASM{wasi_snapshot_preview1} = __FILE__;
+
+    $linker;
+  };
+
+  if(@global)
+  {
+    Carp::croak("Cannot specify both Wasm and -global") if @module;
+    foreach my $spec (@global)
+    {
+      my($name, $content, $mutability, $value) = @$spec;
+      my $global = Wasm::Wasmtime::Global->new(
+        $linker->store,
+        Wasm::Wasmtime::GlobalType->new($content, $mutability),
+        $value,
+      );
+      no strict 'refs';
+      *{"${package}::$name"} = $global->tie;
+    }
+    return;
+  }
+
   @module = (wat => '(module)') unless @module;
 
-  require Wasm::Wasmtime;
-  $store ||= do {
-    my $config = Wasm::Wasmtime::Config->new;
-    $config->wasm_multi_value(1);
-    my $engine = Wasm::Wasmtime::Engine->new($config);
-    Wasm::Wasmtime::Store->new($engine);
-  };
-  my $module = Wasm::Wasmtime::Module->new($store, @module);
-  my $instance = Wasm::Wasmtime::Instance->new($module, \@imports);
+  Carp::croak("Wasm for $package already loaded") if $inst{$package};
+
+  my $module = Wasm::Wasmtime::Module->new($linker->store, @module);
+
+  foreach my $import (@{ $module->imports })
+  {
+
+    my $module = $import->module;
+    next if $module eq 'wasi_snapshot_preview1';
+    if($module ne 'main')
+    {
+      my $pm = "$module.pm";
+      $pm =~ s{::}{/}g;
+      eval { require $pm };
+      if(my $error = $@)
+      {
+        $error =~ s/ at (.*?)$//;
+        $error .= " module required by WebAssembly at $file";
+        Carp::croak("$error");
+      }
+    }
+
+    next if $inst{$module};
+
+    my $name = $import->name;
+    my $type = $import->type;
+    my $kind = $type->kind;
+
+    my $extern;
+
+    if($kind eq 'functype')
+    {
+      if(my $f = $module->can("${module}::$name"))
+      {
+        $extern = Wasm::Wasmtime::Func->new(
+          $linker->store,
+          $type,
+          $f,
+        );
+        push @keep, $extern;
+      }
+    }
+    elsif($kind eq 'globaltype')
+    {
+      if(my $global = do { no strict 'refs'; tied ${"${module}::$name"} })
+      {
+        $extern = $global;
+      }
+    }
+
+    if($extern)
+    {
+      # TODO: check that the store is the same?
+      eval {
+        $linker->define(
+          $module,
+          $name,
+          $extern,
+        );
+      };
+      if(my $error = $@)
+      {
+        if(Wasm::Wasmtime::Error->can('new'))
+        {
+          # TODO: if we can do a get on the define that would
+          # be better than doing this regex on the diagnostic.
+          # this is available in the rust api, but not the c api
+          # as of this writing.
+          die $error unless $error =~ /defined twice/;
+        }
+        else
+        {
+          # TODO: also for the prod version of wasmtime we don't
+          # have an error so we end up swallowing other types
+          # of errors, if there are any.
+        }
+      }
+    }
+  }
+
+  my $instance = $inst{$package} = $linker->instantiate($module);
+  $linker->define_instance($package, $instance);
+  $WASM{$package} = "$file";
 
   my @me = @{ $module->exports   };
   my @ie = @{ $instance->exports };
@@ -120,12 +263,19 @@ sub import
     my $name = $me[$i]->name;
     my $externtype = $exporttype->type;
     my $extern = $ie[$i];
-    if($externtype->kind eq 'functype')
+    my $kind = $extern->kind;
+    if($kind eq 'func')
     {
       my $func = $extern;
       $func->attach($package, $name);
       push @function_names, $name;
     }
+    elsif($kind eq 'global')
+    {
+      my $global = $extern;
+      no strict 'refs';
+      *{"${package}::$name"} = $global->tie;
+    }
   }
 
   if($exporter)
@@ -158,7 +308,7 @@ Wasm - Write Perl extensions using Wasm
 
 =head1 VERSION
 
-version 0.07
+version 0.08
 
 =head1 SYNOPSIS
 
@@ -202,6 +352,10 @@ is under active development.  Use with caution.
 
 The C<Wasm> Perl dist provides tools for writing Perl bindings using WebAssembly (Wasm).
 
+The current implementation uses L<Wasm::Wasmtime>, which is itself based on the Rust project
+Wasmtime.  This module doesn't expose the L<Wasm::Wasmtime> interface, and implementation
+could be changed in the future.
+
 =head1 OPTIONS
 
 =head2 -api
@@ -224,12 +378,6 @@ or C<@EXPORT_OK> (C<ok>).
 
 Path to a WebAssembly file in either WebAssembly Text (.wat) or WebAssembly binary (.wasm) format.
 
-=head2 -imports
-
- use Wasm -api => 0, -imports => \@imports;
-
-Use the given imports when creating the module instance.
-
 =head2 -package
 
  use Wasm -api => 0, -package => $package;
@@ -252,6 +400,13 @@ C<lib/Foo/Bar.wasm>.  If both exist, then it will use the newer of the two.
 
 String containing WebAssembly Text (WAT).  Helpful for inline WebAssembly inside your Perl source file.
 
+=head1 GLOBALS
+
+=head2 %Wasm::WASM
+
+This hash maps the Wasm module names to the files from which the Wasm was loaded.
+It is roughly analogous to the C<@INC> array in Perl.
+
 =head1 CAVEATS
 
 As mentioned before as of this writing this dist is a work in progress.  I won't intentionally break
