@@ -11,7 +11,7 @@ use constant is_global => 1;
 use constant kind => 'global';
 
 # ABSTRACT: Wasmtime global class
-our $VERSION = '0.07'; # VERSION
+our $VERSION = '0.08'; # VERSION
 
 
 $ffi_prefix = 'wasm_global_';
@@ -68,6 +68,19 @@ $ffi->attach( set => ['wasm_global_t','wasm_val_t'] => sub {
   $xsub->($self, $value);
 });
 
+
+sub tie
+{
+  my $self = shift;
+  my $ref;
+  tie $ref, __PACKAGE__, $self;
+  \$ref;
+}
+
+sub TIESCALAR { $_[1] }
+*FETCH = \&get;
+*STORE = \&set;
+
 __PACKAGE__->_cast(1);
 _generate_destroy();
 
@@ -85,7 +98,7 @@ Wasm::Wasmtime::Global - Wasmtime global class
 
 =head1 VERSION
 
-version 0.07
+version 0.08
 
 =head1 SYNOPSIS
 
@@ -140,6 +153,12 @@ Gets the global value.
 
 Sets the global to the given value.
 
+=head2 tie
+
+ my $ref = $global->tie;
+
+Returns a reference to a tied scalar that can be used to get/set the global.
+
 =head1 SEE ALSO
 
 =over 4
