@@ -12,7 +12,7 @@ use Ref::Util qw( is_blessed_ref );
 use Carp ();
 
 # ABSTRACT: Wasmtime linker class
-our $VERSION = '0.07'; # VERSION
+our $VERSION = '0.08'; # VERSION
 
 
 $ffi_prefix = 'wasmtime_linker_';
@@ -180,6 +180,9 @@ else
   });
 }
 
+
+sub store { shift->{store} }
+
 _generate_destroy();
 
 1;
@@ -196,7 +199,7 @@ Wasm::Wasmtime::Linker - Wasmtime linker class
 
 =head1 VERSION
 
-version 0.07
+version 0.08
 
 =head1 SYNOPSIS
 
@@ -324,6 +327,7 @@ Define WASI instance.
 =head2 define_instance
 
  $linker->define_instance(
+   $name,       # string
    $instance,   # Wasm::Wasmtime::Instance
  );
 
@@ -337,6 +341,12 @@ Define WebAssembly instance.
 
 Instantiate the module using the linker.  Returns the new L<Wasm::Wasmtime::Instance> object.
 
+=head2 store
+
+ my $store = $linker->store;
+
+Returns the L<Wasm::Wasmtime::Store> for the linker.
+
 =head1 SEE ALSO
 
 =over 4
